@@ -8,7 +8,7 @@ use vars qw($VERSION);
 use DBI;
 use File::Spec;
 
-$VERSION = "0.11";
+$VERSION = "0.2";
 
 # The database should be in the same directory as this file. Get the location.
 my (undef, $path) = File::Spec->splitpath(__FILE__);
@@ -17,15 +17,6 @@ my $db = $path . 'locale.db';
 # Check it's a binary file in the right location.
 croak "FATAL ERROR: The Locale::Object database was not in '$path', where I expected it. Please check your installation." unless -B $db;
 
-# Make a hash of allowed table names for searches.
-my %allowed_tables = map { $_ => 1 }
-  qw(continent country currency language);
-  
-# Make a hash of allowed column names for searches.
-my %allowed_columns = map { $_ => 1 }
-  qw(country_code name name_native primary_language main_timezone uses_daylight_savings
-  code code_numeric symbol subunit subunit_amount code_alpha2 code_alpha3);  
-
 # Make a new object.
 sub new
 {
@@ -55,15 +46,16 @@ sub lookup
   
   my @results = $dbh->selectrow_array("SELECT * FROM $table WHERE $what=?", {}, $value);
 
-  # Croak if no match.
-  croak "Error: No result when looking in '$table' for a '$what' of '$value'." unless @results;
+  # Return error if no match.
+  return "No result when looking in '$table' for a '$what' of '$value'." unless @results;
   
   return @results;
 }
 
 # Method to return all values of 'result_column' in 'table' in rows that 
 # have 'value' in 'search_column'.
-sub lookup_all
+
+sub lookup
 {
   my ($self, %params) = @_;
 
@@ -73,7 +65,7 @@ sub lookup_all
   # Croak if any of them are missing.
   for (0..$#required)
   {
-    croak "Error: could not do lookup_all: no '$required[$_]' specified." unless defined($params{$required[$_]});
+    croak "Error: could not do lookup_arrayref: no '$required[$_]' specified." unless defined($params{$required[$_]});
   }
 
   # Validate parameters.
@@ -87,16 +79,51 @@ sub lookup_all
   # Execute it.
   $sth->execute($params{value}) or croak "Error: Couldn't execute SQL statement: " . DBI::errstr;
   
-  my @results;
-  
-  # Put each result into @results.
-  while ( my @rows = $sth->fetchrow_array() ) {
-    push @results, $rows[0] or croak "Error: Couldn't fetch database row: " . DBI::errstr;
-  }
-    
-  return @results;
+  # Return a reference to an array of hashes.
+  return $sth->fetchall_arrayref({});
 }
 
+# Get a value for a cell in a row which matches 2 columns and their values specified.
+sub lookup_dual
+{
+  my ($self, %params) = @_;
+
+  # Required parameters for this method.
+  my @required = qw(table result_col col_1 val_1 col_2 val_2);
+  
+  # Croak if any of them are missing.
+  for (0..$#required)
+  {
+    croak "Error: could not do lookup_arrayref: no '$required[$_]' specified." unless defined($params{$required[$_]});
+  }
+
+  # Validate parameters.
+  _check_search_params($params{table}, $params{result_col}, $params{col_1}, $params{val_1}, $params{col_2}, $params{val_2});
+  
+  # Prepare the SQL statement. 
+  my $sth = $dbh->prepare(
+    "SELECT $params{result_col} from $params{table} WHERE $params{col_1}=? AND $params{col_2}=?"
+  ) or croak "Error: Couldn't prepare SQL statement: " . DBI::errstr;
+
+  # Execute it.
+  $sth->execute($params{val_1}, $params{val_2})
+    or croak "Error: Couldn't execute SQL statement: " . DBI::errstr;
+  
+  # Return a reference to an array of hashes.
+  return $sth->fetchall_arrayref({});
+}
+
+
+# Make a hash of allowed table names for searches.
+my %allowed_tables = map { $_ => 1 }
+  qw(continent country currency language language_mappings);
+  
+# Make a hash of allowed column names for searches.
+my %allowed_columns = map { $_ => 1 }
+  qw(country_code name name_native primary_language main_timezone 
+  uses_daylight_savings code code_numeric symbol subunit subunit_amount 
+  code_alpha2 code_alpha3 id country language official *);  
+
 # Sub for sanity check on search parameters. Does nothing except croak if an error is encountered.
 sub _check_search_params
 {
@@ -106,7 +133,7 @@ sub _check_search_params
   croak "Error: $table is not a valid table."
   unless $allowed_tables{$table};
     
-  # Check parameters for lookup_all().
+  # Check parameters.
   if ($result_column)
   {
     # You can only specify a valid column name.  
@@ -128,7 +155,7 @@ Locale::Object::DB - do database lookups for Locale::Object modules
 
 =head1 VERSION
 
-0.11
+0.2 - this refers also to the accompanying database file, and will be incremented if that is updated.
 
 =head1 DESCRIPTION
 
@@ -151,33 +178,58 @@ This module provides common functionality for the Locale::Object modules by doin
     $table = 'continent';
     my $result_column = 'country_code';
     
-    foreach my $result ( $db->lookup_all(
-                                         table => $table, 
-                                         result_column => $result_column, 
-                                         search_column => $what, 
-                                         value => $value) ) {
+    my $results = $db->lookup(
+                              table         => $table, 
+                              result_column => $result_column, 
+                              search_column => $what, 
+                              value         => $value
+                             );
                                            
-      # fill up %countries with countries in the same continent as $value
-      $countries{$result} = 1; 
+    foreach my $item (@{$results})
+    {
+      print $item->{$result_column};
     }
+
+    $result = $db->lookup_dual(
+                               table      => $table, 
+                               result_col => $result_column, 
+                               col_1      => $first_search_column, 
+                               val_1      => $first_search_value,
+                               col_2      => $second_search_column,
+                               val_2      => $second_search_value
+                              );
     
 =head1 METHODS
 
-=head2 lookup()
+=head2 C<lookup()>
 
-    $db->lookup($table, $what, $value);
+    $db->lookup(
+                table         => $table,
+                result_column => $result_column,
+                search_column => $search_column,
+                value         => $value
+               );
     
-C<lookup> will return an array of results from a query of the database for a row matching the table name, column name and cell value that you specify. If no match was found, a message saying so will be returned. Any NULL values in matching rows will be returned in the results as UNDEF.
-
-=head2 lookup_all()
-
-    $db->lookup_all( table => $table, result_column => $result_column,
-                     search_column => $search_column, value => $value );
-    
-C<lookup_all> will return an array of results for a query of the database for cells in $result_column in $table that are in a row that has $value in $search_column.
+C<lookup> will return a reference to an anonymous array of hashes. The hashes will contain the results for a query of the database for cells in $result_column in $table that are in a row that has $value in $search_column. Use '*' as a value for result_column if you want to retrieve whole rows.
 
 For information on what db tables are available and where the data came from, see L<Locale::Object::DB::Schemata>.
 
+IMPORTANT: The way of using this method has changed as of version 0.2. It now supersedes the place formerly taken by C<lookup_all()>, in addition to being used in its former fashion. Apologies for any inconvenience.
+
+=head2 C<lookup_dual()>
+
+    my $result = $db->lookup_dual(
+                                  table      => $table, 
+                                  result_col => $result_column, 
+                                  col_1      => $first_search_column, 
+                                  val_1      => $first_search_value,
+                                  col_2      => $second_search_column,
+                                  val_2      => $second_search_value
+                                 );
+
+C<lookup_dual> will return a reference to an anonymous array of hashes. The hashes will contain the results for a query of the database for cells in $result_column in $table that are in a row that has two specified values in two specified columns. Use '*' as a value for result_column if you want to retrieve whole rows.
+
+                                 
 =head1 NOTES
 
 The database file itself is named C<locale.db> and must reside in the same directory as this module. If it's not present, the module will croak with a fatal error.
